C=======================================================================
C     UTILITIES AND FUNCTIONS FOR MODPREDICT
C     - UTILITIES ALL BEGIN WITH LETTER "U"
C     - FUNCTIONS HAVE VARIOUS SYNTAX
C=======================================================================

c=======================================================================
      SUBROUTINE UWTCALC(IFAIL,HVAR,IVAR,SVAR,EVAR,WTVAR,N)
c=======================================================================
C     CALCULATE AN ENTRY FOR THE WEIGHT MATRIX - PASS BACK THE ORIGINAL
C     VALUES OF HVAR,IVAR,SVAR,EVAR IN CASE THE CALLING ROUTINE IS
C     WRITING THEM OUT TO A NEW FILE

      USE DRIVER
      IMPLICIT NONE

      INTEGER IFAIL,IVAR,N
      REAL HVAR,SVAR,EVAR,WTVAR,oHVAR,oSVAR,oEVAR

      IFAIL = 0

c     STORE PASSED VARIABLES
      oHVAR =HVAR
      oSVAR =SVAR
      oEVAR =EVAR
      IF (HVAR.EQ.0 .AND. IVAR.EQ.2) THEN
        WRITE (IOUTP,605) N
        STOP
      ENDIF

      IF (IVAR.EQ.0) WTVAR = 1./(SVAR*EVAR)
      IF (IVAR.EQ.1) WTVAR = 1./(SVAR*SVAR*EVAR)
      IF (IVAR.EQ.2) WTVAR = 1./(((SVAR*HVAR)**2.)*EVAR)
      IF (WTVAR.LE.0.) THEN
        IFAIL = 1
        WRITE (IOUTP,560) N
        STOP
      ENDIF

c     RESTORE PASSED VARIABLES
      HVAR =oHVAR
      SVAR =oSVAR
      EVAR =oEVAR

      RETURN

560   FORMAT (/,' FOR OBS ',I6,' STATISTIC RELATED TO WEIGHT < OR =0',/,
     &        ' -- STOP EXECUTION (UWTCALC)',/)
605   FORMAT (/,' OBSERVATION ',I6,' EQUALS ZERO, THE STATISTIC ',
     &        'CAN NOT BE A',/,' COEFFICIENT OF VARIATION (ISTAT=2)',
     &        ' -- STOP EXECUTION (UWTCALC)')

      END

C=======================================================================
      SUBROUTINE URDCOM(IN,IOUTP,LINE)
C=======================================================================
C-----VERSION 02FEB1999 URDCOM
C     READ COMMENTS FROM A FILE AND PRINT THEM.  RETURN THE FIRST LINE
C     THAT IS NOT A COMMENT
C     IF IOUTP PASSED AS ZERO THEN PRINTING SUPRESSED
C=======================================================================

      CHARACTER*(*) LINE

   10 READ(IN,'(A)') LINE
c      write(*,*) line
      IF(LINE(1:1).NE.'#') RETURN
      L=LEN(LINE)
      IF(L.GT.79) L=79
      DO 20 I=L,1,-1
      IF(LINE(I:I).NE.' ') GO TO 30
   20 CONTINUE
   30 IF (IOUTP.GT.0) WRITE(IOUTP,'(3X,A)') LINE(1:I)
      GO TO 10

C     RETURN??
      END

C=======================================================================
      SUBROUTINE UCASE(WORDIN,WORDOUT,ICASE)
C=======================================================================
C-----VERSION 19980316 ERB
C     CONVERT A CHARACTER STRING TO ALL UPPER (ICASE > 0) OR ALL
C     LOWER (ICASE < 0) CASE.  IF ICASE = 0, NO CASE CONVERSION IS DONE.
C=======================================================================
C     SPECIFICATIONS:

      CHARACTER WORDIN*(*), WORDOUT*(*)
      INTEGER ICASE, LENGNB, LENGIN, LENGOUT

C     DETERMINE LENGTH OF STRING VARIABLES
      LENGIN = LEN(WORDIN)
      LENGOUT = LEN(WORDOUT)

C     DETERMINE IF WORDOUT IS LONG ENOUGH TO CONTAIN NON-BLANK LENGTH
C     OF WORDIN
      LENGNB = NONB_LEN(WORDIN,LENGIN)
      IF (LENGNB.GT.LENGOUT) STOP 'STRING-LENGTH ERROR IN UCASE'
C
      WORDOUT = WORDIN
      IDIFF=ICHAR('a')-ICHAR('A')
      IF (ICASE.GT.0) THEN
C       CONVERT STRING TO UPPER CASE
        DO 10 K=1,LENGNB
          IF(WORDIN(K:K).GE.'a' .AND. WORDIN(K:K).LE.'z')
     1      WORDOUT(K:K)=CHAR(ICHAR(WORDIN(K:K))-IDIFF)
10      CONTINUE
C
      ELSEIF (ICASE.LT.0) THEN
C       CONVERT STRING TO LOWER CASE
        DO 20 K=1,LENGNB
          IF(WORDIN(K:K).GE.'A' .AND. WORDIN(K:K).LE.'Z')
     1      WORDOUT(K:K)=CHAR(ICHAR(WORDIN(K:K))+IDIFF)
20      CONTINUE
C
      ENDIF
C
      RETURN
      END

C=======================================================================
      SUBROUTINE UTEXT2INT(IFAIL,TEXT,II)
C=======================================================================
C-----VERSION 20030109
C     CONVERT A CHARACTER VARIABLE TO AN INTEGER
C=======================================================================
      INTEGER IFAIL,II
      CHARACTER*(*) TEXT
      CHARACTER*6 AFMT

      IFAIL=0
      AFMT='(I   )'
      WRITE(AFMT(3:5),'(I3)') LEN(TEXT)
      READ(TEXT,AFMT,ERR=100) II
      RETURN

  100 IFAIL=1
      RETURN
      END

c=======================================================================
      SUBROUTINE URWORD(LINE,ICOL,ISTART,ISTOP,NCODE,N,R,IOUT,IN)
c=======================================================================
C-----VERSION 1003 05AUG1992 URWORD
C
C     ROUTINE TO EXTRACT A WORD FROM A LINE OF TEXT, AND OPTIONALLY
C     CONVERT THE WORD TO A NUMBER.
c=======================================================================

      CHARACTER*(*) LINE
      CHARACTER*20 RW,STRING
      CHARACTER*1 TAB
      INTEGER ISTART,ISTOP,NCODE,N,IOUT,IN,ICOL,LINLEN
      TAB=CHAR(9)
C
C ----Set last char in LINE to blank and set ISTART and ISTOP to point
C ----to this blank as a default situation when no word is found.  If
C ----starting location in LINE is out of bounds, do not look for a word

      LINLEN=LEN(LINE)
      LINE(LINLEN:LINLEN)=' '
      ISTART=LINLEN
      ISTOP=LINLEN
      LINLEN=LINLEN-1
      IF(ICOL.LT.1 .OR. ICOL.GT.LINLEN) GO TO 100
C
C2------Find start of word, which is indicated by first character that
C2------is not a blank, a comma, or a tab.
      DO 10 I=ICOL,LINLEN
      IF(LINE(I:I).NE.' ' .AND. LINE(I:I).NE.','
     &    .AND. LINE(I:I).NE.TAB) GO TO 20
10    CONTINUE
      ICOL=LINLEN+1
      GO TO 100
C
C3------Found start of word.  Look for end.
C3A-----When word is quoted, only a quote can terminate it.
20    IF(LINE(I:I).EQ.'''') THEN
         I=I+1
         IF(I.LE.LINLEN) THEN
            DO 25 J=I,LINLEN
            IF(LINE(J:J).EQ.'''') GO TO 40
25          CONTINUE
         END IF
C
C3B-----When word is not quoted, space, comma, or tab will terminate.
      ELSE
         DO 30 J=I,LINLEN
         IF(LINE(J:J).EQ.' ' .OR. LINE(J:J).EQ.','
     &    .OR. LINE(I:I).EQ.TAB) GO TO 40
30       CONTINUE
      END IF
C
C3C-----End of line without finding end of word; set end of word to
C3C-----end of line.
      J=LINLEN+1
C
C4------Found end of word; set J to point to last character in WORD and
C-------set ICOL to point to location for scanning for another word.
40    ICOL=J+1
      J=J-1
      IF(J.LT.I) GO TO 100
      ISTART=I
      ISTOP=J
C
C5------Convert word to upper case and RETURN if NCODE is 1.
      IF(NCODE.EQ.1) THEN
         IDIFF=ICHAR('a')-ICHAR('A')
         DO 50 K=ISTART,ISTOP
            IF(LINE(K:K).GE.'a' .AND. LINE(K:K).LE.'z')
     1             LINE(K:K)=CHAR(ICHAR(LINE(K:K))-IDIFF)
50       CONTINUE
         RETURN
      END IF
C
C6------Convert word to a number if requested.
100   IF(NCODE.EQ.2 .OR. NCODE.EQ.3) THEN
         RW=' '
         L=20-ISTOP+ISTART
         IF(L.LT.1) GO TO 200
         RW(L:20)=LINE(ISTART:ISTOP)
         IF(NCODE.EQ.2) READ(RW,'(I20)',ERR=200) N
         IF(NCODE.EQ.3) READ(RW,'(F20.0)',ERR=200) R
      END IF
      RETURN
C
C7------Number conversion error.
200   IF(NCODE.EQ.3) THEN
         STRING= 'A REAL NUMBER'
         L=13
      ELSE
         STRING= 'AN INTEGER'
         L=10
      END IF
C
C7A-----If output unit is negative, set last character of string to 'E'.
      IF(IOUT.LT.0) THEN
         N=0
         R=0.
         LINE(LINLEN+1:LINLEN+1)='E'
         RETURN
C
C7B-----If output unit is positive; write a message to output unit.
      ELSE IF(IOUT.GT.0) THEN
         IF(IN.GT.0) THEN
            WRITE(IOUT,201) IN,LINE(ISTART:ISTOP),STRING(1:L),LINE
         ELSE
            WRITE(IOUT,202) LINE(ISTART:ISTOP),STRING(1:L),LINE
         END IF
201      FORMAT(1X,/1X,'FILE UNIT ',I4,' : ERROR CONVERTING "',A,
     1       '" TO ',A,' IN LINE:',/1X,A)
202      FORMAT(1X,/1X,'KEYBOARD INPUT : ERROR CONVERTING "',A,
     1       '" TO ',A,' IN LINE:',/1X,A)
C
C7C-----If output unit is 0; write a message to default output.
      ELSE
         IF(IN.GT.0) THEN
            WRITE(*,201) IN,LINE(ISTART:ISTOP),STRING(1:L),LINE
         ELSE
            WRITE(*,202) LINE(ISTART:ISTOP),STRING(1:L),LINE
         END IF
      END IF
C
C7D-----STOP after writing message.
      STOP
      END

c=======================================================================
      subroutine Udiffname(ifail,num,aname,iname)
c=======================================================================
C -- Subroutine Udiffname makes names different if any are the same
c=======================================================================

        USE DRIVER
        implicit none

        integer num,iname(num),ifail,iloop,ichange,icount,i,j,
     +  ispace,istart,nb,nblnk,ii
        character*1 apad(5)
        character*12 aname(num),tname,anum

        data apad /'-','@','~','_','&'/

        ifail=0
        iloop=0
        ii=1

10      iloop=iloop+1
        do i=1,num
          iname(i)=0
        end do

        ichange=0
        do i=1,num-1
          if(iname(i).ne.0)go to 200
          tname=aname(i)
          icount=0
          do j=i+1,num
            if(iname(j).ne.0)go to 100
            if(aname(j).eq.tname)then
              icount=icount+1
              if(icount.eq.1)iname(i)=i
              iname(j)=i
            end if
100         continue
          end do
          if(icount.eq.0) go to 200
          if(icount.gt.99999999)then
            write(errmsg,110)
110         format('Too many observations with the same name in ',
     +      'MODFLOW input dataset.')
            call UWRTERR(1)
            ifail=1
            return
          else if(icount.gt.9999999)then
            ispace=9
          else if(icount.gt.999999)then
            ispace=8
          else if(icount.gt.99999)then
            ispace=7
          else if(icount.gt.9999)then
            ispace=6
          else if(icount.gt.999)then
            ispace=5
          else if(icount.gt.99)then
            ispace=4
          else if(icount.gt.9)then
            ispace=3
          else
            ispace=2
          end if
          istart=12-ispace+1
          nb=nblnk(tname)
          if(istart.gt.nb+1)istart=nb+1

          call Uwritint(anum,1)
          aname(i)(istart:)=apad(ii)//trim(anum)
          icount=1
          do j=i+1,num
            if(iname(j).eq.i)then
              icount=icount+1
              call Uwritint(anum,icount)
              aname(j)(istart:)=apad(ii)//trim(anum)
            end if
          end do
          ichange=ichange+1
          if(iloop.gt.1)then
            ii=ii+1
            if(ii.gt.5)ii=1
          end if
200       continue
        end do

        if(ichange.gt.0)then
          if(iloop.ge.5)then
            write(errmsg,210)
210         format('Difficulties encountered in re-naming observations',
     +      'or prior information to give them non-identical names. ',
     +      'Rename them yourself, giving them all different names. ',
     +      'Try shortening their names.')
            call UWRTERR(1)
            ifail=2
            return
          else
            go to 10
          end if
        else
          return
        end if

      end

C=======================================================================
      SUBROUTINE URUNMF2K(FN,RUNTYPE,RUNSTRING,IFAIL,BASENAME)
C=======================================================================
C     RUNS THE APPROPRIATE MODFLOW CASE, AND REPORTS SOME MESSAGES TO
C     THE MOD-PREDICT OUTPUT FILE
C=======================================================================

      USE DRIVER
      IMPLICIT NONE

      CHARACTER*200 FN,MCOMMAND,RUNSTRING,BASENAME
      INTEGER IFAIL,RUNTYPE

      WRITE (*,101) 'RUNNING',TRIM(ADJUSTL(RUNSTRING)),TRIM(FN)
      WRITE (IOUTP,101) 'RUNNING',TRIM(ADJUSTL(RUNSTRING)),TRIM(FN)
      MCOMMAND = ' '
      MCOMMAND = TRIM(MF2KCOM)//' '//TRIM(FN)//' > NUL'
      CALL SYSTEM(MCOMMAND)
      MCOMMAND = '(COMMAND = '//TRIM(ADJUSTL(MCOMMAND))//')'
      WRITE(IOUTP,*) MCOMMAND

      CALL CHECKRUN(IFAIL,RUNTYPE,BASENAME)
      IF (IFAIL.NE.0) THEN
          WRITE(IOUTP,199)
          GOTO 999
      END IF
      WRITE (*,101) ' - COMPLETED',TRIM(ADJUSTL(RUNSTRING)),TRIM(FN)
      WRITE (IOUTP,101) ' - COMPLETED',TRIM(ADJUSTL(RUNSTRING)),TRIM(FN)

      RETURN

101   FORMAT(/,1X,A,' ',A,T50,A)
199   FORMAT(/,1X,'ERROR RUNNING MODFLOW CASE - "URUNMF2K" ')
999   RETURN

      END

C=======================================================================
      SUBROUTINE CHECKRUN(IFAIL,RUNTYPE,BASENAME)
C=======================================================================
C     CHECK MODFLOW RAN OKAY -
C      RELIES ON SUCCESSFULLY OPENING AN UNDERSCORE FILE
C=======================================================================

      USE DRIVER
      IMPLICIT NONE

      CHARACTER*200 BASENAME,FN
      INTEGER IFAIL,RUNTYPE,IGETUNIT,II,IERR

      IFAIL = 0
      II = IGETUNIT(1,95)
      IF (RUNTYPE.EQ.1) THEN
          FN = TRIM(ADJUSTL(BASENAME))//OBSSUF(1)
        ELSE
          FN = TRIM(ADJUSTL(BASENAME))//SENSUF(1)
      ENDIF
      WRITE(IOUTP,*) ' - CHECKING FOR ',FN
      OPEN(II,FILE=TRIM(FN),STATUS='OLD',IOSTAT=IERR)
      IF (IERR.NE.0) THEN
        WRITE(IOUTP,98)
        IFAIL = 1
        RETURN
      END IF

98    FORMAT(//,1X,'ERROR READING MODFLOW UNDERSCORE FILES - CHECK TO ',
     +             'CONFIRM MODFLOW EXECUTED CORRECLTY')
      WRITE(IOUTP,99)
99    FORMAT(1X,' - CHECK RUN OKAY')

      RETURN
      END

c=======================================================================
        SUBROUTINE PGETCL(COMLIN)
c=======================================================================

        CHARACTER COMLIN*(*)

        CALL GETCL(COMLIN)
        RETURN
        END

c=======================================================================
        subroutine Uaddquote(afile)
c=======================================================================

          integer nblnk,nb
          character*(*) afile

          if(afile(1:1).eq.'"') return
          nb=nblnk(afile)
          if(index(afile(1:nb),' ').ne.0)then
            afile='"'//afile(1:nb)//'"'
          end if

          return

        end

c=======================================================================
         SUBROUTINE ULINESPLIT(IFAIL,NUM,LW,RW,CLINE)
c=======================================================================
C -- Subroutine ULINESPLIT splits a string into blank-delimited fragments.
c=======================================================================

        INTEGER IFAIL,NW,NBLC,J,I
        INTEGER NUM
C        ,NBLNK !NOT NEEDED WITH LAHEY: USE LAHEY FUNCTION NBLANK
        INTEGER LW(NUM),RW(NUM)
        CHARACTER*(*) CLINE

        IFAIL=0
        NW=0
c        NBLC=NBLNK(CLINE)
        NBLC=NBLaNK(CLINE)
        IF((NBLC.NE.0).AND.(INDEX(CLINE,CHAR(9)).NE.0)) THEN
          CALL UTABREM(CLINE)
c          NBLC=NBLNK(CLINE)
          NBLC=NBLaNK(CLINE)
        ENDIF
        IF(NBLC.EQ.0) THEN
          IFAIL=-1
          RETURN
        END IF
        J=0
5       IF(NW.EQ.NUM) RETURN
        DO 10 I=J+1,NBLC
          IF((CLINE(I:I).NE.' ').AND.(CLINE(I:I).NE.',').AND.
     +    (ICHAR(CLINE(I:I)).NE.9)) GO TO 20
10      CONTINUE
        IFAIL=1
        RETURN
20      NW=NW+1
        LW(NW)=I
        DO 30 I=LW(NW)+1,NBLC
          IF((CLINE(I:I).EQ.' ').OR.(CLINE(I:I).EQ.',').OR.
     +    (ICHAR(CLINE(I:I)).EQ.9)) GO TO 40
30      CONTINUE
        RW(NW)=NBLC
        IF(NW.LT.NUM) IFAIL=1
        RETURN
40      RW(NW)=I-1
        J=RW(NW)
        GO TO 5

        END

c=======================================================================
        SUBROUTINE UTABREM(CLINE)
c=======================================================================
C -- Subroutine UTABREM removes tabs from a string.
c=======================================================================

        INTEGER I
        CHARACTER*(*) CLINE

        DO 10 I=1,LEN(CLINE)
10      IF(ICHAR(CLINE(I:I)).EQ.9) CLINE(I:I)=' '

        RETURN
        END

c=======================================================================
        SUBROUTINE UREALREAD(IFAIL,CLINE,RTEMP)
c=======================================================================
C -- Subroutine UREALREAD reads a real number from a string.
c=======================================================================

        INTEGER IFAIL
        real RTEMP
        CHARACTER*8 AFMT
        CHARACTER*(*) CLINE

        IFAIL=0
        AFMT='(F   .0)'
        WRITE(AFMT(3:5),'(I3)') nblank(CLINE)
        READ(CLINE,AFMT,ERR=100) RTEMP
        RETURN

100     IFAIL=1
        RETURN
        END

c=======================================================================
        SUBROUTINE INTREAD(IFAIL,CLINE,ITEMP)
c=======================================================================
C -- Subroutine intREAD reads an integer from a string.
c=======================================================================

        INTEGER IFAIL
        integer iTEMP
        CHARACTER*6 AFMT
        CHARACTER*(*) CLINE

        IFAIL=0
        AFMT='(i   )'
        WRITE(AFMT(3:5),'(I3)') LEN(CLINE)
        READ(CLINE,AFMT,ERR=100) iTEMP
        RETURN

100     IFAIL=1
        RETURN
        END

c=======================================================================
        subroutine Ufinddelim(ifail,delim,nplist,parnam)
c=======================================================================

          integer ifail,nplist
          integer i,j
          character delim
          character*(*) parnam(nplist)
          character trychar(8)

          data trychar /'$','%','^','&','*','@','~','|'/

          ifail=0
          do i=1,8
            do j=1,nplist
              if(index(parnam(j),trychar(i)).ne.0) go to 10
            end do
            delim=trychar(i)
            return
10          continue
          end do
          ifail=1
          return

        end

c=======================================================================
      Subroutine Ugetfile(ifail,cline,filename,ibeg,iend)
c=======================================================================
C -- Subroutine UGETFILE extracts a filename from a string.
C    (Note that this is also used in MODFLOW2000, so if any changes
C     are made here then they must be made there as well.)
c=======================================================================

        integer ibeg,iend,ifail,i,j,k
        character*(*) cline,filename
        character*1 aa

        ifail=0
        do 30 i=ibeg,iend
          aa=cline(i:i)
          if((aa.ne.' ').and.(aa.ne.',').and.(aa.ne.char(9)))go to 50
30      continue
        ifail=1
        return

50      if((aa.eq.'"').or.(aa.eq.''''))then
          do j=i+1,iend
            if(cline(j:j).eq.aa) go to 60
          end do
          ifail=1
          return
60        iend=j
          if(i+1.gt.j-1)then
            ifail=1
            return
          else
            filename=cline(i+1:j-1)
          end if
        else
          do j=i+1,iend
            if((cline(j:j).eq.' ').or.
     +         (cline(j:j).eq.',').or.(cline(j:j).eq.char(9)))then
              k=j-1
              go to 100
            end if
          end do
          k=iend
100       filename=cline(i:k)
          iend=k
        end if
        call Ulshift(filename)
        return

      end

c=======================================================================
      SUBROUTINE UWRTERR(itype)
c=======================================================================
C -- SUBROUTINE UWRTERR FORMATS AND WRITES ERROR MESSAGES
c=======================================================================

        USE DRIVER
        IMPLICIT NONE

        INTEGER NBLNK,itype
        INTEGER IERR,J,JEND,I,NBLC,ITAKE,MCASE

        numerr=numerr+1
        if(itype.eq.1)then
          write(IOUTP,2)
2         format(/,' ',78('*'))
          write(IOUTP,3)
3         format(/,' Error condition prevents continued ',
     +    'execution:-',/)
        end if

        ITAKE=0
        J=0
        NBLC=NBLNK(ERRMSG)
5       JEND=J+78-ITAKE
        IF(JEND.GE.NBLC) GO TO 100
        DO 10 I=JEND,J+1,-1
        IF(ERRMSG(I:I).EQ.' ') THEN
          IF(ITAKE.EQ.0) THEN
            WRITE(IOUTP,'(1X,A)',ERR=200) ERRMSG(J+1:I)
            ITAKE=3
          ELSE
             WRITE(IOUTP,'(1X,A)',ERR=200) ERRMSG(J+1:I)
          END IF
          J=I
          GO TO 5
        END IF
10      CONTINUE
        IF(ITAKE.EQ.0)THEN
          WRITE(IOUTP,'(1X,A)',ERR=200) ERRMSG(J+1:JEND)
          ITAKE=3
        ELSE
           WRITE(IOUTP,'(1X,A)',ERR=200) ERRMSG(J+1:JEND)
        END IF
        J=JEND
        GO TO 5
100     JEND=NBLC
        IF(ITAKE.EQ.0)THEN
          WRITE(IOUTP,'(1X,A)',ERR=200) ERRMSG(J+1:JEND)
        ELSE
           WRITE(IOUTP,'(1X,A)',ERR=200) ERRMSG(J+1:JEND)
        END IF

        if(itype.eq.0)then
          RETURN
        else
          write(IOUTP,2)
          call EXIT(100)
        end if

200     return
      END

c=======================================================================
      subroutine Ucasetrans(string,hi_or_lo)
c=======================================================================
C -- Subroutine casetrans converts a string to upper or lower case.
c=======================================================================

        USE DRIVER
        IMPLICIT NONE

        character*(*) string,hi_or_lo
        character alo, ahi
        integer inc,i,nblnk

        if(hi_or_lo.eq.'lo') then
          alo='A'
          ahi='Z'
          inc=iachar('a')-iachar('A')
        else if(hi_or_lo.eq.'hi') then
          alo='a'
          ahi='z'
          inc=iachar('A')-iachar('a')
        else
          write(errmsg,10)
10        format('Error calling subroutine UCASETRANS.')
          call UWRTERR(1)
        endif

        do 20 i=1,nblnk(string)
          if((string(i:i).ge.alo).and.(string(i:i).le.ahi))
     +    string(i:i)=achar(iachar(string(i:i))+inc)
20      continue

        return

      end

c=======================================================================
      subroutine Ulshift(cline)
c=======================================================================
C -- Subroutine LSHIFT left-justifies a character string.
c=======================================================================

      character*(*) cline
      cline=adjustl(cline)
      return
      end

c=======================================================================
      subroutine Uwritlin(aline,iline,afile)
c=======================================================================
C -- Subroutine WRITLIN writes a string containing the line number of
C    a file.
c=======================================================================

        integer iline,nblnk
        character*(*) aline,afile
        character*10 atemp

        call Uwritint(atemp,iline)
        aline='Line '//atemp(1:nblnk(atemp))//' of file '//
     +  afile(1:nblnk(afile))//': '

        return

      end

c=======================================================================
      subroutine Uwritint(anum,inum)
c=======================================================================
C --  Subroutine WRITINT writes an integer to a character variable.
c=======================================================================

        integer*4 inum
        character*6 afmt
        character*(*) anum

        afmt='(i   )'
        write(afmt(3:5),'(i3)') len(anum)
        write(anum,afmt)inum
        call Ulshift(anum)
        return
      end

c=======================================================================
      subroutine Ureadvar(ifail,itype,aname,ivar,rvar,avar,ic1,
     +ic2,afile,iline)
c=======================================================================
C -- Subroutine UREADVAR reads a variable from a string.
c=======================================================================

        USE DRIVER
        IMPLICIT NONE

        integer itype,ivar,ic1,ic2,iline,ifail,nblnk
        real rvar
        character*150 aline
        character*(*) aname,avar,afile

C     0 = INTEGER
C     1 = REAL

        ifail=0

        if((itype.eq.0).or.(itype.eq.1))then
          call Unumrd(ifail,itype,ic1,ic2,ivar,rvar,cline)
          if(ifail.ne.0) go to 100
        else
          avar=cline(ic1:ic2)
        end if
        return

100     continue
        ifail=1
        call Uwritlin(aline,iline,afile)
        write(errmsg,110) aline(1:nblnk(aline)+1),
     +  aname(1:nblnk(aname))
110     format(a,'error reading variable ',a,'.')
        call UWRTERR(1)
        return

      end

c=======================================================================
      subroutine Unumrd(ifail,itype,ic1,ic2,ivar,rvar,cline)
c=======================================================================
C -- Subroutine NUMRD reads a real number or integer from a string.
c=======================================================================

        integer ifail,itype,ic1,ic2,ivar
        real rvar
        character*(*) cline

        ifail=0
        if(itype.eq.0)then
          call intread(ifail,cline(ic1:ic2),ivar)
        else
          call Urealread(ifail,cline(ic1:ic2),rvar)
        end if

        return
      end

c=======================================================================
      subroutine unexpectval(avar,iline,infile)
c=======================================================================

        USE DRIVER
        IMPLICIT NONE

        integer iline,nblnk
        character*(*) avar,infile
        character*150 aline

        call Uwritlin(aline,iline,infile)
        write(errmsg,10) aline(1:nblnk(aline)+1),avar(1:nblnk(avar))
10      format(a,': unexpected ',a,' variable.')
        call UWRTERR(1)
        return

      end

C=======================================================================
      SUBROUTINE UCLOSEALL(IUNIT,NIUNIT)
C=======================================================================
      INTEGER NIUNIT,IUNIT(NIUNIT)
      LOGICAL LOP

      DO I=1,NIUNIT
          IF (IUNIT(I).NE.0) THEN
              INQUIRE(UNIT=IUNIT(I),OPENED=LOP)
              IF (LOP) CLOSE(IUNIT(I))
          END IF
      END DO

      RETURN
      END

C=======================================================================
      SUBROUTINE U2DREL(A,ANAME,II,JJ,K,IN,IOUT)
C=======================================================================
C
C-----VERSION 1539 22JUNE1993 U2DREL
C     ******************************************************************
C     ROUTINE TO INPUT 2-D REAL DATA MATRICES
C       A IS ARRAY TO INPUT
C       ANAME IS 24 CHARACTER DESCRIPTION OF A
C       II IS NO. OF ROWS
C       JJ IS NO. OF COLS
C       K IS LAYER NO. (USED WITH NAME TO TITLE PRINTOUT --)
C              IF K=0, NO LAYER IS PRINTED
C              IF K<0, CROSS SECTION IS PRINTED)
C       IN IS INPUT UNIT
C       IOUT IS OUTPUT UNIT
C     ******************************************************************
C
C        SPECIFICATIONS:
C     ------------------------------------------------------------------
      CHARACTER*24 ANAME
      DIMENSION A(JJ,II)
      CHARACTER*20 FMTIN
      CHARACTER*200 CNTRL
      CHARACTER*16 TEXT
      CHARACTER*200 FNAME
      DATA NUNOPN/99/
      INCLUDE 'openspec.inc'
C     ------------------------------------------------------------------
C
C1------READ ARRAY CONTROL RECORD AS CHARACTER DATA.
      READ(IN,'(A)') CNTRL
C
C2------LOOK FOR ALPHABETIC WORD THAT INDICATES THAT THE RECORD IS FREE
C2------FORMAT.  SET A FLAG SPECIFYING IF FREE FORMAT OR FIXED FORMAT.
      ICLOSE=0
      IFREE=1
      ICOL=1
      CALL URWORD(CNTRL,ICOL,ISTART,ISTOP,1,N,R,IOUT,IN)
      IF (CNTRL(ISTART:ISTOP).EQ.'CONSTANT') THEN
         LOCAT=0
      ELSE IF(CNTRL(ISTART:ISTOP).EQ.'INTERNAL') THEN
         LOCAT=IN
      ELSE IF(CNTRL(ISTART:ISTOP).EQ.'EXTERNAL') THEN
         CALL URWORD(CNTRL,ICOL,ISTART,ISTOP,2,LOCAT,R,IOUT,IN)
      ELSE IF(CNTRL(ISTART:ISTOP).EQ.'OPEN/CLOSE') THEN
         CALL URWORD(CNTRL,ICOL,ISTART,ISTOP,0,N,R,IOUT,IN)
         FNAME=CNTRL(ISTART:ISTOP)
         LOCAT=NUNOPN
         WRITE(iout,15) LOCAT,FNAME
   15    FORMAT(1X,/1X,'OPENING FILE ON UNIT ',I4,':',/1X,A)
         ICLOSE=1
      ELSE
C
C2A-----DID NOT FIND A RECOGNIZED WORD, SO NOT USING FREE FORMAT.
C2A-----READ THE CONTROL RECORD THE ORIGINAL WAY.
         IFREE=0
         READ(CNTRL,1,ERR=500) LOCAT,CNSTNT,FMTIN,IPRN
    1    FORMAT(I10,F10.0,A20,I10)
      END IF
C
C3------FOR FREE FORMAT CONTROL RECORD, READ REMAINING FIELDS.
      IF(IFREE.NE.0) THEN
         CALL URWORD(CNTRL,ICOL,ISTART,ISTOP,3,N,CNSTNT,iout,IN)
         IF(LOCAT.NE.0) THEN
            CALL URWORD(CNTRL,ICOL,ISTART,ISTOP,1,N,R,iout,IN)
            FMTIN=CNTRL(ISTART:ISTOP)
            IF(ICLOSE.NE.0) THEN
               IF(FMTIN.EQ.'(BINARY)') THEN
                  OPEN(UNIT=LOCAT,FILE=FNAME,FORM=FORM,ACCESS=ACCESS,
     &                 ACTION=ACTION(1))
               ELSE
                  OPEN(UNIT=LOCAT,FILE=FNAME,ACTION=ACTION(1))
               END IF
            END IF
            IF(LOCAT.GT.0 .AND. FMTIN.EQ.'(BINARY)') LOCAT=-LOCAT
            CALL URWORD(CNTRL,ICOL,ISTART,ISTOP,2,IPRN,R,iout,IN)
         END IF
      END IF
C
C4------TEST LOCAT TO SEE HOW TO DEFINE ARRAY VALUES.
      IF(LOCAT) 200,50,90
C
C4A-----LOCAT=0; SET ALL ARRAY VALUES EQUAL TO CNSTNT. RETURN.
   50 DO 80 I=1,II
      DO 80 J=1,JJ
   80 A(J,I)=CNSTNT
c      IF(K.GT.0) WRITE(iout,2) ANAME,CNSTNT,K
c    2 FORMAT(1X,/1X,A,' =',1P,G14.6,' FOR LAYER',I4)
c      IF(K.LE.0) WRITE(iout,3) ANAME,CNSTNT
c    3 FORMAT(1X,/1X,A,' =',1P,G14.6)
      RETURN
C
C4B-----LOCAT>0; READ FORMATTED RECORDS USING FORMAT FMTIN.
   90 CONTINUE
      IF(K.GT.0) THEN
         WRITE(iout,94) ANAME,K,LOCAT,FMTIN
   94    FORMAT(1X,///11X,A,' FOR LAYER',I4,/
     1    1X,'READING ON UNIT ',I4,' WITH FORMAT: ',A)
      ELSE IF(K.EQ.0) THEN
         WRITE(iout,95) ANAME,LOCAT,FMTIN
   95    FORMAT(1X,///11X,A,/
     1    1X,'READING ON UNIT ',I4,' WITH FORMAT: ',A)
      ELSE
         WRITE(iout,96) ANAME,LOCAT,FMTIN
   96    FORMAT(1X,///11X,A,' FOR CROSS SECTION',/
     1    1X,'READING ON UNIT ',I4,' WITH FORMAT: ',A)
      END IF
      DO 100 I=1,II
      IF(FMTIN.EQ.'(FREE)') THEN
         READ(LOCAT,*) (A(J,I),J=1,JJ)
      ELSE
         READ(LOCAT,FMTIN) (A(J,I),J=1,JJ)
      END IF
  100 CONTINUE
      GO TO 300
C
C4C-----LOCAT<0; READ UNFORMATTED ARRAY VALUES.
  200 LOCAT=-LOCAT
      IF(K.GT.0) THEN
         WRITE(iout,201) ANAME,K,LOCAT
  201    FORMAT(1X,///11X,A,' FOR LAYER',I4,/
     1    1X,'READING BINARY ON UNIT ',I4)
      ELSE IF(K.EQ.0) THEN
         WRITE(iout,202) ANAME,LOCAT
  202    FORMAT(1X,///1X,A,/
     1    1X,'READING BINARY ON UNIT ',I4)
      ELSE
         WRITE(iout,203) ANAME,LOCAT
  203    FORMAT(1X,///1X,A,' FOR CROSS SECTION',/
     1    1X,'READING BINARY ON UNIT ',I4)
      END IF
      READ(LOCAT) KSTP,KPER,PERTIM,TOTIM,TEXT,NCOL,NROW,ILAY
      READ(LOCAT) A
C
C5------IF CNSTNT NOT ZERO THEN MULTIPLY ARRAY VALUES BY CNSTNT.
  300 IF(ICLOSE.NE.0) CLOSE(UNIT=LOCAT)
      ZERO=0.
      IF(CNSTNT.EQ.ZERO) GO TO 320
      DO 310 I=1,II
      DO 310 J=1,JJ
      A(J,I)=A(J,I)*CNSTNT
  310 CONTINUE
C
C6------IF PRINT CODE (IPRN) >0 OR =0 THEN PRINT ARRAY VALUES.
C  320 IF(IPRN.GE.0) CALL ULAPRW(A,ANAME,0,0,JJ,II,0,IPRN,iout)
  320 CONTINUE
C
C7------RETURN
      RETURN
C
C8------CONTROL RECORD ERROR.
  500 IF(K.GT.0) THEN
         WRITE(iout,501) ANAME,K
  501    FORMAT(1X,/1X,'ERROR READING ARRAY CONTROL RECORD FOR ',A,
     1     ' FOR LAYER',I4,':')
      ELSE
         WRITE(iout,502) ANAME
  502    FORMAT(1X,/1X,'ERROR READING ARRAY CONTROL RECORD FOR ',A,':')
      END IF
      WRITE(iout,'(1X,A)') CNTRL
      STOP
      END

C=======================================================================
      SUBROUTINE ULAPRWC1(A,NCOL,NROW,ILAY,IOUT,IPRN,ANAME)
C=======================================================================
C
C-----VERSION 01JULY1998 ULAPRWC
C     ******************************************************************
C     CHECK TO SEE IF AN ARRAY IS CONSTANT, AND PRINT IT APPROPRIATELY
C     ******************************************************************
C
C     SPECIFICATIONS:
C=======================================================================
      IMPLICIT NONE
      INTEGER NCOL,NROW,ILAY,i,j,iprn,iout,IIN
      REAL A,TMP
      DIMENSION A(NCOL,NROW)
      CHARACTER*(*) ANAME
C=======================================================================
C
C  Check to see if entire array is a constant.
      TMP=A(1,1)
      IIN = 0
      DO 300 I=1,NROW
      DO 300 J=1,NCOL
      IF(A(J,I).NE.TMP) GO TO 400
  300 CONTINUE
C      IF(ILAY.GT.0) THEN
         WRITE(IOUT,304) IIN,TMP
304      format(I10,1x,1PG9.4)
      RETURN
C
C  Print the array.
  400 IF(ILAY.GT.0) THEN
         WRITE(IOUT,494) ANAME,ILAY
  494    FORMAT(1X,//11X,A,' FOR LAYER',I4)
      ELSE IF(ILAY.EQ.0) THEN
         WRITE(IOUT,495) ANAME
  495    FORMAT(1X,//11X,A)
      ELSE
         WRITE(IOUT,496) ANAME
  496    FORMAT(1X,//11X,A,' FOR CROSS SECTION')
      END IF
C
      RETURN
      END

c=======================================================================
      SUBROUTINE UWRTSIG(IFAIL,VAL,WORD,NW,PRECIS,TVAL,NOPNT)
c=======================================================================
C  SUBROUTINE WRTSIG WRITES A NUMBER INTO A CONFINED SPACE WITH MAXIMUM
C  PRECISION
C  (Author: J. Doherty)
c=======================================================================
C
c  Revision history:
C     July, 1993: version 1.
C     August 1994: modified for unix version (#ifdef's added)
C     August, 1995: #ifdefs commented out for inclusion in Groundwater
C                   Data Utilities
c
C  Failure criteria:
C         ifail= 1 ...... number too large or small for single precision type
C         ifail= 2 ...... number too large or small for double precision type
C         ifail= 3 ...... field width too small to represent number
C         ifail=-1 ...... internal error type 1
C         ifail=-2 ...... internal error type 2
C         ifail=-3 ...... internal error type 3
c=======================================================================

      INTEGER PRECIS,LW,POS,INC,D,P,W,J,JJ,K,JEXP,N,JFAIL,NW
      integer EPOS,PP,NOPNT,KEXP,IFLAG,LEXP
	INTEGER IFAIL
      real VAL,TVAL
	CHARACTER*29 TWORD,TTWORD,FMT*14
	CHARACTER*(*) WORD

      LEXP=0
	IFLAG=0
	WORD=' '
	POS=1
	IF(VAL.LT.0.0D0)POS=0
!#ifdef USE_D_FORMAT
!        WRITE(TWORD,'(1PD23.15D3)') VAL
!#else
	WRITE(TWORD,'(1PE23.15E3)') VAL
!#endif
C      WRITE(*,*) TWORD

	READ(TWORD(20:23),'(I4)') JEXP
	EPOS=1
	IF(JEXP.LT.0)EPOS=0

	JFAIL=0
	IFAIL=0
	IF(PRECIS.EQ.0)THEN
	  LW=MIN(15,NW)
	ELSE
	  LW=MIN(23,NW)
	END IF

	N=0
	IF(NOPNT.EQ.1)N=N+1
	IF(POS.EQ.1)N=N+1
	IF(PRECIS.EQ.0)THEN
	  IF(ABS(JEXP).GT.38)THEN
	    IFAIL=1
	    RETURN
	  END IF
	  IF(POS.EQ.1) THEN
	    IF(LW.GE.13) THEN
	      WRITE(WORD,'(1PE13.7)',ERR=80) VAL
	      GO TO 200
	    END IF
	  ELSE
	    IF(LW.GE.14)THEN
	      WRITE(WORD,'(1PE14.7)',ERR=80) VAL
	      GO TO 200
	    END IF
	  END IF
	  IF(LW.GE.14-N) THEN
	    LW=14-N
	    GO TO 80
	  END IF
	ELSE
	  IF(ABS(JEXP).GT.275)THEN
	    IFAIL=2
	    RETURN
	  END IF
	  IF(POS.EQ.1) THEN
	    IF(LW.GE.22) THEN
!#ifdef USE_D_FORMAT
!              WRITE(WORD,'(1PD22.15D3)',ERR=80) VAL
!#else
	      WRITE(WORD,'(1PE22.15E3)',ERR=80) VAL
!#endif
	      GO TO 200
	    END IF
	  ELSE
	    IF(LW.GE.23) THEN
!#ifdef USE_D_FORMAT
!              WRITE(WORD,'(1PD23.15D3)',ERR=80) VAL
!#else
	      WRITE(WORD,'(1PE23.15E3)',ERR=80) VAL
!#endif
	      GO TO 200
	    END IF
	  END IF
	  IF(LW.GE.23-N)THEN
	    LW=23-N
	    GO TO 80
	  END IF
	END IF

	IF(NOPNT.EQ.1)THEN
	  IF((JEXP.EQ.LW-2+POS).OR.(JEXP.EQ.LW-3+POS))THEN
	    WRITE(FMT,15)LW+1
15          FORMAT('(F',I2,'.0)')
	    WRITE(WORD,FMT,ERR=19) VAL
	    IF(INDEX(WORD,'*').NE.0) GO TO 19
	    IF(WORD(1:1).EQ.' ') GO TO 19
	    WORD(LW+1:LW+1)=' '
	    GO TO 200
	  END IF
	END IF
19      D=MIN(LW-2+POS,LW-JEXP-3+POS)
20      IF(D.LT.0) GO TO 80
	WRITE(FMT,30) LW,D
30      FORMAT('(F',I2,'.',I2,')')
	WRITE(WORD,FMT,ERR=80) VAL
	IF(INDEX(WORD,'*').NE.0) THEN
	  D=D-1
	  GO TO 20
	END IF
	K=INDEX(WORD,'.')
	IF(K.EQ.0)THEN
	  IFAIL=-1
	  RETURN
	END IF
	IF((K.EQ.1).OR.((POS.EQ.0).AND.(K.EQ.2)))THEN
	  DO 70 J=1,3
	  IF(K+J.GT.LW) GO TO 75
	  IF(WORD(K+J:K+J).NE.'0') GO TO 200
70        CONTINUE
	  GO TO 80
75        IFAIL=3
	  RETURN
	END IF
	GO TO 200

80      WORD=' '
	IF(NOPNT.EQ.0)THEN
	  D=LW-7
	  IF(POS.EQ.1) D=D+1
	  IF(EPOS.EQ.1) D=D+1
	  IF(ABS(JEXP).LT.100) D=D+1
	  IF(ABS(JEXP).LT.10) D=D+1
	  IF((JEXP.GE.100).AND.(JEXP-(D-1).LT.100))THEN
	    P=1+(JEXP-99)
	    D=D+1
	    LEXP=99
	  ELSE IF((JEXP.GE.10).AND.(JEXP-(D-1).LT.10))THEN
	    P=1+(JEXP-9)
	    D=D+1
	    LEXP=9
	  ELSE IF((JEXP.EQ.-10).OR.(JEXP.EQ.-100)) THEN
	    IFLAG=1
	    D=D+1
	  ELSE
	    P=1
	  END IF
	  INC=0
85        IF(D.LE.0) GO TO 300
	  IF(IFLAG.EQ.0)THEN
	    WRITE(FMT,100,ERR=300) P,D+7,D-1
	  ELSE
	    WRITE(FMT,100,ERR=300) 0,D+8,D
	  END IF
	  WRITE(TWORD,FMT) VAL
	  IF(IFLAG.EQ.1) GO TO 87
	  READ(TWORD(D+4:D+7),'(I4)',ERR=500) KEXP
        IF(((KEXP.EQ.10).AND.((JEXP.EQ.9).OR.(LEXP.EQ.9))).OR.
     +  ((KEXP.EQ.100).AND.((JEXP.EQ.99).OR.LEXP.EQ.99))) THEN
	    IF(INC.EQ.0)THEN
	      IF(LEXP.EQ.0)THEN
		IF(D-1.EQ.0) THEN
		  D=D-1
		ELSE
		  P=P+1
		END IF
	      ELSE IF(LEXP.EQ.9)THEN
		IF(JEXP-(D-2).LT.10) THEN
		  P=P+1
		ELSE
		  D=D-1
		END IF
	      ELSE IF(LEXP.EQ.99)THEN
		IF(JEXP-(D-2).LT.100)THEN
		  P=P+1
		ELSE
		  D=D-1
		END IF
	      END IF
	      INC=INC+1
	      GO TO 85
	    END IF
	  END IF
!#ifdef USE_D_FORMAT
!87        J=INDEX(TWORD,'D')
!#else
87        J=INDEX(TWORD,'E')
!#endif
	  GO TO 151
	END IF
	INC=0
	P=LW-2
	PP=JEXP-(P-1)
	IF(PP.GE.10)THEN
	  P=P-1
	  IF(PP.GE.100)P=P-1
	ELSE IF(PP.LT.0)THEN
	  P=P-1
	  IF(PP.LE.-10)THEN
	    P=P-1
	    IF(PP.LE.-100)P=P-1
	  END IF
	END IF
	IF(POS.EQ.0)P=P-1
90      CONTINUE
	D=P-1
	W=D+8
	WRITE(FMT,100) P,W,D
	IF(D.LT.0)THEN
	  IF(JFAIL.EQ.1) GO TO 300
	  JFAIL=1
	  P=P+1
	  GO TO 90
	END IF
!#ifdef USE_D_FORMAT
!100     FORMAT('(',I2,'pD',I2,'.',I2,'D3)')
!#else
100     FORMAT('(',I2,'pE',I2,'.',I2,'E3)')
!#endif
	WRITE(TWORD,FMT) VAL
!#ifdef USE_D_FORMAT
!        J=INDEX(TWORD,'D')
!#else
	J=INDEX(TWORD,'E')
!#endif
	IF(TWORD(J-1:J-1).NE.'.')THEN
	  IFAIL=-1
	  RETURN
	END IF
	N=1
	IF(TWORD(J+1:J+1).EQ.'-') N=N+1
	IF(TWORD(J+2:J+2).NE.'0') THEN
	  N=N+2
	  GO TO 120
	END IF
	IF(TWORD(J+3:J+3).NE.'0') N=N+1
120     N=N+1
	IF(J+N-2-POS.LT.LW)THEN
	  IF(INC.EQ.-1) GO TO 150
	  TTWORD=TWORD
	  P=P+1
	  INC=1
	  GO TO 90
	ELSE IF(J+N-2-POS.EQ.LW) THEN
	  GO TO 150
	ELSE
	  IF(INC.EQ.1)THEN
	    TWORD=TTWORD
	    GO TO 150
	  END IF
	  IF(JFAIL.EQ.1) GO TO 300
	  P=P-1
	  INC=-1
	  GO TO 90
	END IF

150     J=INDEX(TWORD,'.')
151     IF(POS.EQ.0)THEN
	  K=1
	ELSE
	 K=2
	END IF
	WORD(1:J-K)=TWORD(K:J-1)
	JJ=J
	J=J-K+1
	IF(PRECIS.EQ.0)THEN
	  WORD(J:J)='E'
	ELSE
	  WORD(J:J)='D'
	END IF
	JJ=JJ+2
	IF(NOPNT.EQ.0) JJ=JJ-1
	IF(TWORD(JJ:JJ).EQ.'-')THEN
	  J=J+1
	  WORD(J:J)='-'
	END IF
	IF(TWORD(JJ+1:JJ+1).NE.'0')THEN
	  J=J+2
	  WORD(J-1:J)=TWORD(JJ+1:JJ+2)
	  GO TO 180
	END IF
	IF(TWORD(JJ+2:JJ+2).NE.'0')THEN
	  J=J+1
	  WORD(J:J)=TWORD(JJ+2:JJ+2)
	END IF
180   J=J+1
	WORD(J:J)=TWORD(JJ+3:JJ+3)
	IF(IFLAG.EQ.1)THEN
	  IF(POS.EQ.1)THEN
	    JJ=1
	  ELSE
	    JJ=2
	  END IF
	  N=len_trim(WORD)
	  DO 190 J=JJ,N-1
190       WORD(J:J)=WORD(J+1:J+1)
	  WORD(N:N)=' '
	END IF

200     IF(len_trim(WORD).GT.LW)THEN
	  IFAIL=-2
	  RETURN
	END IF
	WRITE(FMT,30) LW,0
	READ(WORD,FMT,ERR=400) TVAL
	RETURN
300     IFAIL=3
	RETURN
400     IFAIL=-3
	RETURN
500     IFAIL=-2
	RETURN
	END

C=======================================================================
      SUBROUTINE UDELOLD(FN)
C=======================================================================
C     CHECKS FOR EXISTING UNDERSCORE FILES AND DELETES THEM
C=======================================================================
      USE DRIVER
      IMPLICIT NONE

      CHARACTER*200 FN
      INTEGER IU,IERR,IGETUNIT

      IU = IGETUNIT(1,95)
      OPEN(IU,FILE=TRIM(FN),STATUS='OLD',IOSTAT=IERR)
      IF (IERR.EQ.0) THEN
        CLOSE(IU)
        WRITE(IOUTP,*) '  DELETING - ',TRIM(FN)
        CALL SYSTEM('DEL '//TRIM(FN))
      END IF

      RETURN
      END

C=======================================================================
      INTEGER FUNCTION IGETUNIT(IFIRST,MAXUNIT)
C=======================================================================
C     VERSION 19981030 ERB
C     FIND FIRST UNUSED FILE UNIT NUMBER BETWEEN IFIRST AND MAXUNIT
C     SPECIFICATIONS:
C
      INTEGER I, IFIRST, IOST, MAXUNIT
      LOGICAL LOP

      LOP = .TRUE.
C
C     LOOP THROUGH RANGE PROVIDED TO FIND FIRST UNUSED UNIT NUMBER
      DO 10 I=IFIRST,MAXUNIT
        INQUIRE(UNIT=I,IOSTAT=IOST,OPENED=LOP,ERR=5)
        IF (IOST.EQ.0) THEN
          IF (.NOT.LOP) THEN
            IGETUNIT = I
            RETURN
          ENDIF
        ENDIF
 5      CONTINUE
10    CONTINUE
C
C     IF THERE ARE NO UNUSED UNIT NUMBERS IN RANGE PROVIDED, RETURN
C     A VALUE INDICATING AN ERROR
      IGETUNIT = -1
C
      RETURN
      END

C=======================================================================
      INTEGER FUNCTION NONB_LEN1(CHARVAR)
C=======================================================================
C     FUNCTION TO RETURN NON-BLANK LENGTH OF CONTENTS OF A CHARACTER
C     VARIABLE. UNLIKE THE NONB_LEN FUNCTION, THE LENGTH OF THE CHARACTER
C     STRING IS NOT SUPPLIED; RATHER IT IS EVALUATED WITHIN THE FUNCTION
C     ******************************************************************
C        SPECIFICATIONS:
C     ------------------------------------------------------------------
C
C Variable list:
C   CHARVAR  = CHARACTER VARIABLE OF INTEREST
C
      CHARACTER*(*) CHARVAR,C*1
C     ------------------------------------------------------------------
C
      LENGTH=LEN(CHARVAR)
      C = ' '
      K = LENGTH+1
C
      DO 20 WHILE (C.EQ.' ' .AND. K.GE.1)
        K = K-1
        C = CHARVAR(K:K)
 20   CONTINUE
C
      NONB_LEN1 = K
C
      RETURN
      END

c=======================================================================
      INTEGER FUNCTION NONB_LEN(CHARVAR,LENGTH)
c=======================================================================
C     FUNCTION TO RETURN NON-BLANK LENGTH OF CONTENTS OF A CHARACTER
C     VARIABLE
c=======================================================================
C
C     CHARVAR  = CHARACTER VARIABLE OF INTEREST
C     LENGTH   = DIMENSIONED LENGTH OF CHARVAR
C
      CHARACTER*(*) CHARVAR,C*1
      INTEGER LENGTH
C
      C = ' '
      K = LENGTH+1
C
      DO 20 WHILE (C.EQ.' ' .AND. K.GE.1)
        K = K-1
        C = CHARVAR(K:K)
 20   CONTINUE
C
      NONB_LEN = K
C
      RETURN
      END

c=======================================================================
      integer function nblnk(aline)
c=======================================================================

        character*(*) aline
        nblnk=len_trim(aline)
        return
      end

c=======================================================================
      integer function getlines(ifail,afile)
c=======================================================================

        USE DRIVER
        IMPLICIT NONE

        integer ierr,ifail,nblnk
        character*(*) afile
        character aachar

        ifail=0
        open(unit=12,file=afile,status='old',iostat=ierr)
        if(ierr.ne.0)then
          write(errmsg,10) afile(1:nblnk(afile))
10        format('Cannot open file ',a,'.')
          call UWRTERR(1)
          ifail=1
          return
        end if
        getlines=0
        do
          getlines=getlines+1
          read(12,'(a)',end=50,err=100) aachar
        end do

50      continue
        close(unit=21)
        return

100     write(errmsg,110) afile(1:nblnk(afile))
110     format('Cannot read file ',a,'.')
        ifail=1
        call UWRTERR(1)
        return
      end

c=======================================================================
      logical function match(aname1,astring1)
c=======================================================================

        integer lenstring,j,i
        character*(*) aname1, astring1
        character*50 aname,astring                                      !protects against bounds errors

        aname=aname1
        astring=astring1
        match=.false.
        if(aname.eq.astring) then
          match=.true.
          return
        end if
        j=0
        i=0
10        i=i+1
          j=j+1
          if(astring(i:i).eq.aname(j:j)) go to 20
          if(astring(i:i).eq.'?')then
            if(aname(j:j).eq.' ')then
              go to 50
            else
              go to 10
            end if
          else if(astring(i:i).eq.'*')then
            if(aname(j:j).eq.' ')go to 50
            i=i+1
15          j=j+1
            if(astring(i:i).eq.' ')go to 40
16          continue
            if(astring(i:i).eq.'*')then
              i=i+1
              go to 16
            end if
            if(aname(j:j).eq.astring(i:i))then
              go to 10
            else if(aname(j:j).eq.' ')then
              go to 50
            else
              go to 15
            end if
          else
            go to 50
          end if
20        continue
          if(astring(i:i).eq.' ')then
            if(aname(j:j).eq.' ')then
              go to 40
            else
              go to 50
            end if
          end if
        go to 10

40      match=.true.
        return
50      match=.false.
        return
      end

c=======================================================================
      logical function equals(rtemp1,rtemp2)
c=======================================================================

        real rtemp1,rtemp2

        equals=.false.
        if(rtemp1.eq.0.0)then
          if(rtemp2.eq.0.0)then
            equals=.true.
            return
          else
            equals=.false.
          end if
        else if(rtemp2.eq.0.0)then
          equals=.false.
          return
        end if
        if(abs(rtemp2-rtemp1).le.abs(rtemp1+rtemp2)*1.0e-5)then
          equals=.true.
        else
          equals=.false.
        end if
        return
      end

C=======================================================================
      SUBROUTINE SVD (IA,A,AS,DTLA,BUFF1,BUFF3,IAAR)
C-----VERSION 20000615 MCH
C     CALCULATE THE INVERSE AND THE SQUARE-ROOT OF THE INVERSE OF A
C     SQUARE MATRIX USING SVD (I.E. DECOMPOSE THE MATRIX INTO A MATRIX
C     OF EIGENVECTORS AND A DIAGONAL MATRIX OF CORRESPONDING
C     EIGENVALUES).
C     V is used as temporary storage of the off-diagonal elements of
C     the returned (by SSVD1) tridiagonal matrix used by
C     SSVD2
C     ******************************************************************
C        SPECIFICATIONS:
C     ------------------------------------------------------------------
      REAL DTLA, A, AS
      INTEGER I, J, IA, IAAR
      DOUBLE PRECISION BUFF1(IAAR,IAAR), BUFF3(IAAR)
      DIMENSION A(IAAR,IAAR), AS(IAAR,IAAR)
C     ADDED BY MJT TO ORIGINAL
      REAL V(IAAR)

C     INITIALIZE
      BUFF1 = 0.
      BUFF3 = 0.

C     STORE IN BUFF
      DO 30 I = 1,IA
        DO 20 J = 1, IA
          BUFF1(J,I) = DBLE(A(J,I))
   20   CONTINUE
   30 CONTINUE

      CALL SSVD1(BUFF1,IA,IA,BUFF3,V)
      CALL SSVD2(BUFF3,V,IA,IA,BUFF1)
C     CALCULATE INVERSE MATRIX
      CALL SSVD3(IA,BUFF1,BUFF3,A)
C     CALCULATE LOG-DETERMINANT OF THE MATRIX
      DTLA = 0.
      DO 40 I = 1, IA
        DTLA = DTLA + SNGL(DLOG(BUFF3(I)))
   40 CONTINUE

C     MJT RETURN
c      RETURN

C     CALCULATE SQUARE-ROOT OF THE INVERSE MATRIX
      DO 50 I = 1, IA
        BUFF3(I) = DSQRT(BUFF3(I))
   50 CONTINUE
      CALL SSVD3(IA,BUFF1,BUFF3,AS)

      RETURN
      END
C=======================================================================
      SUBROUTINE SSVD1(A,N,NP,D,E)
C-----VERSION 1000 01FEB1992
C
C     ******************************************************************
C     COPYRIGHT (C) 1986 NUMERICAL RECIPES SOFTWARE, TRED2S,
C     REPRODUCED BY PERMISSION FROM THE BOOK NUMERICAL RECIPES: THE ART
C     OF SCIENTIFIC COMPUTING, PUBLISHED BY CAMBRIDGE UNIVERSITY PRESS
C     MODIFIED FOR DOUBLE PRECISION
C     ******************************************************************
C        SPECIFICATIONS:
C     ------------------------------------------------------------------
      DOUBLE PRECISION A, D, F, G, H, HH, SCALE
      INTEGER I, J, K, L, N, NP
      DIMENSION A(NP,NP), D(NP)
      REAL E(NP)
C     ------------------------------------------------------------------
      IF (N.GT.1) THEN
        DO 80 I = N, 2, -1
          L = I - 1
          H = 0.0
          SCALE = 0.0
          IF (L.GT.1) THEN
            DO 10 K = 1, L
              SCALE = SCALE + DABS(A(I,K))
   10       CONTINUE
            IF (SCALE.EQ.0.0) THEN
              E(I) = A(I,L)
            ELSE
              DO 20 K = 1, L
                A(I,K) = A(I,K)/SCALE
                H = H + A(I,K)**2
   20         CONTINUE
              F = A(I,L)
              G = -SIGN(DSQRT(H),F)
              E(I) = SCALE*G
              H = H - F*G
              A(I,L) = F - G
              F = 0.0
              DO 50 J = 1, L
                A(J,I) = A(I,J)/H
                G = 0.0
                DO 30 K = 1, J
                  G = G + A(J,K)*A(I,K)
   30           CONTINUE
                IF (L.GT.J) THEN
                  DO 40 K = J+1, L
                    G = G + A(K,J)*A(I,K)
   40             CONTINUE
                ENDIF
                E(J) = G/H
                F = F + E(J)*A(I,J)
   50         CONTINUE
              HH = F/(H+H)
              DO 70 J = 1, L
                F = A(I,J)
                G = E(J) - HH*F
                E(J) = G
                DO 60 K = 1, J
                  A(J,K) = A(J,K) - F*E(K) - G*A(I,K)
   60           CONTINUE
   70         CONTINUE
            ENDIF
          ELSE
            E(I) = A(I,L)
          ENDIF
          D(I) = H
   80   CONTINUE
      ENDIF
      D(1) = 0.0
      E(1) = 0.0
      DO 130 I = 1, N
        L = I - 1
        IF (D(I).NE.0.0) THEN
          DO 110 J = 1, L
            G = 0.0
            DO 90 K = 1, L
              G = G + A(I,K)*A(K,J)
   90       CONTINUE
            DO 100 K = 1, L
              A(K,J) = A(K,J) - G*A(K,I)
  100       CONTINUE
  110     CONTINUE
        ENDIF
        D(I) = A(I,I)
        A(I,I) = 1.0
        IF (L.GE.1) THEN
          DO 120 J = 1, L
            A(I,J) = 0.0
            A(J,I) = 0.0
  120     CONTINUE
        ENDIF
  130 CONTINUE
      RETURN
      END
C=======================================================================
      SUBROUTINE SSVD2(D,E,N,NP,Z)
C-----VERSION 1000 01FEB1992
C
C     ******************************************************************
C     COPYRIGHT (C) 1986 NUMERICAL RECIPES SOFTWARE, TQLIS,
C     REPRODUCED BY PERMISSION FROM THE BOOK NUMERICAL RECIPES: THE ART
C     OF SCIENTIFIC COMPUTING, PUBLISHED BY CAMBRIDGE UNIVERSITY PRESS
C     MODIFIED FOR DOUBLE PRECISION
C     ******************************************************************
C        SPECIFICATIONS:
C     ------------------------------------------------------------------
      DOUBLE PRECISION B, C, D, DD, F, G, P, R, S, Z
      INTEGER I, ITER, K, L, M, N, NP
      CHARACTER*1 CHR
      DIMENSION D(NP), E(NP), Z(NP,NP)
C     ------------------------------------------------------------------
      IF (N.GT.1) THEN
        DO 10 I = 2, N
          E(I-1) = E(I)
   10   CONTINUE
        E(N) = 0.0
        DO 70 L = 1, N
          ITER = 0
   20     DO 30 M = L, N-1
            DD = DABS(D(M)) + DABS(D(M+1))
            IF (ABS(E(M))+DD.EQ.DD) GOTO 40
   30     CONTINUE
          M = N
   40     IF (M.NE.L) THEN
            IF (ITER.EQ.30) THEN
              WRITE (*,*) ' TOO MANY ITERATIONS'
              READ (*,'(A)') CHR
            ENDIF
            ITER = ITER + 1
            G = (D(L+1)-D(L))/(2.0*E(L))
            R = DSQRT(G**2+1.0)
            G = D(M) - D(L) + E(L)/(G+SIGN(R,G))
            S = 1.0
            C = 1.0
            P = 0.0
            DO 60 I = M-1, L, -1
              F = S*E(I)
              B = C*E(I)
              IF (DABS(F).GE.DABS(G)) THEN
                C = G/F
                R = DSQRT(C**2+1.0)
                E(I+1) = F*R
                S = 1.0/R
                C = C*S
              ELSE
                S = F/G
                R = DSQRT(S**2+1.0)
                E(I+1) = G*R
                C = 1.0/R
                S = S*C
              ENDIF
              G = D(I+1) - P
              R = (D(I)-G)*S + 2.0*C*B
              P = S*R
              D(I+1) = G + P
              G = C*R - B
              DO 50 K = 1, N
                F = Z(K,I+1)
                Z(K,I+1) = S*Z(K,I) + C*F
                Z(K,I) = C*Z(K,I) - S*F
   50         CONTINUE
   60       CONTINUE
            D(L) = D(L) - P
            E(L) = G
            E(M) = 0.0
            GOTO 20
          ENDIF
   70   CONTINUE
      ENDIF
      RETURN
      END
C=======================================================================
      SUBROUTINE SSVD3(N,A,C,B)
C-----VERSION 19990504 ERB
C
C     ******************************************************************
C     FORMERLY SEN1MI
C     CALCULATE THE INVERSE (B) OF A SYMMETRIC MATRIX WITH THE EIGEN-
C     VECTORS STORED IN A AND THE EIGENVALUES IN C BY
C     B = A * C-1 * A(TRANSPOSE)
C=======================================================================
      REAL B, TMP
      INTEGER I, J, K, N
      DOUBLE PRECISION A(N,N), C(N)
      DIMENSION B(N,N)
C=======================================================================
      DO 20 I = 1, N
        DO 10 J = 1, N
          B(I,J) = A(I,J)/C(J)
   10   CONTINUE
   20 CONTINUE
      DO 50 I = 1, N
        DO 40 J = I, N
          TMP = 0.0
          DO 30 K = 1, N
            TMP = TMP + B(J,K)*A(I,K)
   30     CONTINUE
          B(I,J) = TMP
   40   CONTINUE
   50 CONTINUE
      DO 70 I = 1, N
        DO 60 J = I + 1, N
          B(J,I) = B(I,J)
   60   CONTINUE
   70 CONTINUE
      RETURN
      END

C=======================================================================
      SUBROUTINE UPARPM(BUF,NPE,IPRC,IOUT,PARNAMTEMP,NPDIM)
C=======================================================================
C-----VERSION 19980825 ERB
C     ******************************************************************
C     PRINT ONE NPE*NPE CORRELATION OR VARIANCE-COVARIANCE MATRIX
C     ******************************************************************
C
C        SPECIFICATIONS:
C     ------------------------------------------------------------------
      DOUBLE PRECISION BUF(NPE,NPE)
      INTEGER NPDIM
      CHARACTER*10 PARNAMTEMP(NPDIM)
C      INCLUDE 'param.inc'
C     ------------------------------------------------------------------
C
C2------MAKE SURE THE FORMAT CODE (IPRC) IS VALID
      IP=IPRC
      IF(IP.LT.1 .OR. IP.GT.10) IP=1
C
C3------LABEL COLUMNS WITH PARAMETER NAMES.
      IF(IP.EQ.1) CALL UCOLLBL(1,NPE,0,11,11,IOUT,PARNAMTEMP,NPDIM)
      IF(IP.EQ.2) CALL UCOLLBL(1,NPE,0,10,12,IOUT,PARNAMTEMP,NPDIM)
      IF(IP.EQ.3) CALL UCOLLBL(1,NPE,0,9,13,IOUT,PARNAMTEMP,NPDIM)
      IF(IP.EQ.4) CALL UCOLLBL(1,NPE,0,8,14,IOUT,PARNAMTEMP,NPDIM)
      IF(IP.EQ.5) CALL UCOLLBL(1,NPE,0,8,15,IOUT,PARNAMTEMP,NPDIM)
      IF(IP.EQ.6) CALL UCOLLBL(1,NPE,0,6,11,IOUT,PARNAMTEMP,NPDIM)
      IF(IP.EQ.7) CALL UCOLLBL(1,NPE,0,5,12,IOUT,PARNAMTEMP,NPDIM)
      IF(IP.EQ.8) CALL UCOLLBL(1,NPE,0,5,13,IOUT,PARNAMTEMP,NPDIM)
      IF(IP.EQ.9) CALL UCOLLBL(1,NPE,0,4,14,IOUT,PARNAMTEMP,NPDIM)
      IF(IP.EQ.10 ) CALL UCOLLBL(1,NPE,0,4,15,IOUT,PARNAMTEMP,NPDIM)
C
C4------LOOP THROUGH THE ROWS PRINTING EACH ONE IN ITS ENTIRETY.
      DO 1000 I=1,NPE
C
C------------ FORMAT 11G10.3
        IF (IP.EQ.1) THEN
          WRITE(IOUT,11) PARNAMTEMP(I),(BUF(J,I),J=1,NPE)
   11     FORMAT(1X,A,1X,1PG10.3,10(1X,G10.3):/(11X,11(1X,G10.3)))
C
C------------ FORMAT 10G11.4
        ELSEIF (IP.EQ.2) THEN
          WRITE(IOUT,21) PARNAMTEMP(I),(BUF(J,I),J=1,NPE)
   21     FORMAT(1X,A,1X,1PG11.4,9(1X,G11.4):/(11X,10(1X,G11.4)))
C
C------------ FORMAT 9G12.5
        ELSEIF (IP.EQ.3) THEN
          WRITE(IOUT,31) PARNAMTEMP(I),(BUF(J,I),J=1,NPE)
   31     FORMAT(1X,A,1X,1PG12.5,8(1X,G12.5):/(11X,9(1X,G12.5)))
C
C------------ FORMAT 8G13.6
        ELSEIF (IP.EQ.4) THEN
          WRITE(IOUT,41) PARNAMTEMP(I),(BUF(J,I),J=1,NPE)
   41     FORMAT(1X,A,1X,1PG13.6,7(1X,G13.6):/(11X,8(1X,G13.6)))
C
C------------ FORMAT 8G14.7
        ELSEIF (IP.EQ.5) THEN
          WRITE(IOUT,51) PARNAMTEMP(I),(BUF(J,I),J=1,NPE)
   51     FORMAT(1X,A,1X,1PG14.7,7(1X,G14.7):/(11X,8(1X,G14.7)))
C
C------------ FORMAT 6G10.3
        ELSEIF (IP.EQ.6) THEN
          WRITE(IOUT,61) PARNAMTEMP(I),(BUF(J,I),J=1,NPE)
   61     FORMAT(1X,A,1X,1PG10.3,5(1X,G10.3):/(11X,6(1X,G10.3)))
C
C------------ FORMAT 5G11.4
        ELSEIF (IP.EQ.7) THEN
          WRITE(IOUT,71) PARNAMTEMP(I),(BUF(J,I),J=1,NPE)
   71     FORMAT(1X,A,1X,1PG11.4,4(1X,G11.4):/(11X,5(1X,G11.4)))
C
C------------ FORMAT 5G12.5
        ELSEIF (IP.EQ.8) THEN
          WRITE(IOUT,81) PARNAMTEMP(I),(BUF(J,I),J=1,NPE)
   81     FORMAT(1X,A,1X,1PG12.5,4(1X,G12.5):/(11X,5(1X,G12.5)))
C
C------------ FORMAT 4G13.6
        ELSEIF (IP.EQ.9) THEN
          WRITE(IOUT,91) PARNAMTEMP(I),(BUF(J,I),J=1,NPE)
   91     FORMAT(1X,A,1X,1PG13.6,3(1X,G13.6):/(11X,4(1X,G13.6)))
C
C------------ FORMAT 4G14.7
        ELSEIF (IP.EQ.10) THEN
          WRITE(IOUT,101) PARNAMTEMP(I),(BUF(J,I),J=1,NPE)
  101     FORMAT(1X,A,1X,1PG14.7,3(1X,G14.7):/(11X,4(1X,G14.7)))
C
        ENDIF
 1000 CONTINUE
C
C5------RETURN
      RETURN
      END

C=======================================================================
      SUBROUTINE UCOLLBL(NLBL1,NLBL2,NSPACE,NCPL,NDIG,IOUT,PARNAMTEMP,
     +                   NPDIM)
C=======================================================================
C-----VERSION 19980825 ERB
C     ******************************************************************
C     LABEL THE COLUMNS OF MATRIX PRINTOUT WITH PARAMETER NAMES
C        NLBL1 IS THE START COLUMN (NUMBER)
C        NLBL2 IS THE STOP COLUMN (NUMBER)
C        NSPACE IS NUMBER OF BLANK SPACES TO LEAVE AT START OF LINE
C        NCPL IS NUMBER OF COLUMNS PER LINE
C        NDIG IS NUMBER OF CHARACTERS IN EACH COLUMN FIELD
C        IOUT IS OUTPUT UNIT
C     ******************************************************************
C
C        SPECIFICATIONS:
C     ------------------------------------------------------------------
      CHARACTER*1 DOT, SPACE
      CHARACTER*200 BF

      DATA DOT,SPACE/'.',' '/
      INTEGER NPDIM
      CHARACTER*10 PARNAMTEMP(NPDIM)
C     ------------------------------------------------------------------
C
C1------CALCULATE # OF COLUMNS TO BE PRINTED (NLBL), WIDTH
C1------OF A LINE (NTOT), NUMBER OF LINES (NWRAP).
      WRITE(IOUT,1)
    1 FORMAT(1X)
      NLBL = NLBL2-NLBL1+1
      N = NLBL
      IF(NLBL.GT.NCPL) N = NCPL
      LENPN = LEN(PARNAMTEMP(1))
      MINCOLSP = LENPN+1
      IF (NDIG.LT.MINCOLSP) THEN
        WRITE(IOUT,200) NDIG,MINCOLSP
 200    FORMAT(' SPECIFIED FIELD WIDTH TOO SMALL FOR PARNAMTEMP',
     &         '--STOP EXECUTION (UCOLLBL)',/
     &         ' NDIG = ',I2,'   MINCOLSP = ',I2)
        STOP
      ENDIF
      NTOT = NSPACE+LENPN+N*NDIG
      IND = (NDIG-LENPN-1)/2
      NWRAP = (NLBL-1)/NCPL+1
      J1 = NLBL1-NCPL
      J2 = NLBL1-1
C
C2------BUILD AND PRINT EACH LINE
      DO 40 N=1,NWRAP
C
C3------CLEAR THE BUFFER (BF).
        DO 20 I=1,130
          BF(I:I)=SPACE
   20   CONTINUE
        NBF = MINCOLSP+1+IND-NDIG
C
C4------DETERMINE FIRST (J1) AND LAST (J2) COLUMN # FOR THIS LINE.
        J1=J1+NCPL
        J2=J2+NCPL
        IF(J2.GT.NLBL2) J2=NLBL2
C5------LOAD THE COLUMN LABELS INTO THE BUFFER.
        DO 30 J=J1,J2
          NBF=NBF+NDIG
          NBF2 = NBF+LENPN-1
          BF(NBF:NBF2) = PARNAMTEMP(J)
   30   CONTINUE
C
C6------PRINT THE CONTENTS OF THE BUFFER (I.E. PRINT THE LINE).
        WRITE(IOUT,31) BF(1:NBF2)
   31   FORMAT(1X,A)
C
   40 CONTINUE
C
C7------PRINT A LINE OF DOTS (FOR ESTHETIC PURPOSES ONLY).
   50 CONTINUE
      WRITE(IOUT,51) (DOT,I=1,NTOT)
   51 FORMAT(1X,200A1)
C
C8------RETURN
      RETURN
      END
